/*****************************************************
Chip type           : ATmega8
Program type        : Application
Clock frequency     : 8,000000 MHz
Memory model        : Small
External SRAM size  : 0
Data Stack size     : 256
*****************************************************/

#include	<mega8.h>
#include	"Hw.h"

unsigned char	Ms10Tmr;
unsigned char	FrontSwitchMajority[4];
unsigned char	RelayState;
unsigned char	HwRelayState;
unsigned char	FrontSwitchState;
unsigned char	PrevFrontSwitchState;
unsigned char	DelayValue;
unsigned char	SecTmr;
unsigned char	PowUpTmr;
//
unsigned char	BitCnt;
unsigned char	RecByteI;
unsigned char	RecByte;
//
unsigned char	MidiChannel;
unsigned char	MidiProgramm;
unsigned char	LastReceivedProgramm;

bit	TimFlg;
bit	prgChangeDetectedBefore;
bit	SwitchSettingsAltered;
bit	DataReceived;

// USART Receiver interrupt service routine
interrupt [USART_RXC] void usart_rx_isr(void)
{
	unsigned char	inByte;

	inByte = UCSRA;
	inByte = UDR;

	if ((inByte & 0x80) != 0)
	{	// Command
		if ((inByte & 0xF0) == 0xC0)
		{
			if ((inByte & 0x0F) == MidiChannel)
			{
				prgChangeDetectedBefore = 1;
			}
		}
		else
		{
			prgChangeDetectedBefore = 0;
		}
	}
	else
	{	// Data
		if (prgChangeDetectedBefore != 0)
		{
			prgChangeDetectedBefore = 0;
			MidiProgramm = (inByte & 0x7F);
			PORTD &= ~0x08;
		}
	}
}

// USART Transmitter interrupt service routine
interrupt [USART_TXC] void usart_tx_isr(void)
{
	UCSRB &= ~0x40;
	TIFR = 0x20;	// Clear ICF1
	TIMSK |= 0x20;	// On TICIE1
}

// Timer 0 overflow interrupt service routine
interrupt [TIM0_OVF] void timer0_ovf_isr(void)
{	// 2 mS @ 8MHz
	if (DelayValue != 0)
		DelayValue--;
	if (Ms10Tmr != 0)
		Ms10Tmr--;
	else
	{	// 10 mS @ 8MHz
		Ms10Tmr = 4;
		FrontSwitchMajority[1] = FrontSwitchMajority[2];
		FrontSwitchMajority[2] = FrontSwitchMajority[3];
		FrontSwitchMajority[3] = (PINC & 0x3F) ^ 0x3F;
		FrontSwitchMajority[0] = ((FrontSwitchMajority[2] | FrontSwitchMajority[3]) & FrontSwitchMajority[1])
			| (FrontSwitchMajority[2] & FrontSwitchMajority[3]);
		MidiChannel = (PIND >> 4) ^ 0x0F;
		if (SecTmr != 0)
			SecTmr--;
		else
		{
			SecTmr = 99;
			if (PowUpTmr < 254)
				PowUpTmr++;
		}
		TimFlg = 1;
	}
}
//TIMSK/TIFR
//Bit 5  TICIE1
//Bit 4  OCIE1A
// Timer 1 input capture interrupt service routine
interrupt [TIM1_CAPT] void timer1_capt_isr(void)
{
	OCR1A = ICR1 + 384;
	BitCnt = 0;
	TIMSK &= ~0x20;	// Off TICIE1
	TIFR = 0x10;	// Clear OCF1A
	TIMSK |= 0x10;	// On OCIE1A
}

// Timer 1 output compare A interrupt service routine
interrupt [TIM1_COMPA] void timer1_compa_isr(void)
{
	OCR1A += 256;
	BitCnt++;
	if (BitCnt < 9)
	{
		RecByteI = RecByteI >> 1;
		if ((PINB & 0x01) != 0)
			RecByteI |= 0x80;
	}
	else
	{
		TIFR = 0x20;	// Clear ICF1
		TIMSK |= 0x20;	// On TICIE1
		TIMSK &= ~0x10;	// Off OCIE1A
		RecByte = RecByteI;
		DataReceived = 1;
	}
}

unsigned char
EEPROM_read(unsigned short Address)
{
	while(EECR & 0x02)	// Wait for completion of previous write
		;
	EEAR = Address;	// Set up address register
	EECR |= 0x01;	// Start eeprom read by writing EERE
	return EEDR;	// Return data from data register
}

void
EEPROM_write(unsigned short Address, unsigned char Data)
{
	if (EEPROM_read(Address) == Data)
	{
//		NextEeProgReq = 1;
		return;
	}
	EEAR = Address;	// Set up address and data registers
	EEDR = Data;
	#asm("cli")
	EECR |= 0x04;	// Write logical one to EEMWE
	EECR |= 0x0A;	// Start eeprom write by setting EEWE
	#asm("sei")
}

void
Wdog(void)
{
	#asm("wdr")
}

void
Waitms(unsigned char Del)
{
	DelayValue = Del;
	while (DelayValue != 0)
	{
		Wdog();
	}
}

void
SetHwRelays(void)
{
	
	HwRelayState = RelayState | 0x01;
	if (PowUpTmr > 10)
		HwRelayState |= OUT6;
	if (PowUpTmr > 15)
		HwRelayState |= OUT7;
	PORTB = HwRelayState;
}
void
SetRelays(void)
{
	SetHwRelays();
	UCSRA = 0x40;
	UCSRB |= 0x40;
	TIMSK &= ~0x20;	// off TICIE1
	TIMSK &= ~0x10;	// Off OCIE1A
	UDR = HwRelayState;
}

void
SetOut1(void)
{
	RelayState |= OUT1;
	SetHwRelays();
	Waitms(5);
	RelayState &= ~(OUT2 | OUT3 | OUT4);
	SetRelays();
	SwitchSettingsAltered = 1;
}

void
SetOut2(void)
{
	RelayState |= OUT2;
	SetHwRelays();
	Waitms(5);
	RelayState &= ~(OUT1 | OUT3 | OUT4);
	SetRelays();
	SwitchSettingsAltered = 1;
}

void
SetOut3(void)
{
	RelayState |= OUT3;
	SetHwRelays();
	Waitms(5);
	RelayState &= ~(OUT1 | OUT2 | OUT4);
	SetRelays();
	SwitchSettingsAltered = 1;
}

void
SetOut4(void)
{
	RelayState |= OUT4;
	SetHwRelays();
	Waitms(5);
	RelayState &= ~(OUT1 | OUT2 | OUT3);
	SetRelays();
	SwitchSettingsAltered = 1;
}

void
SetOut5(void)
{
	RelayState ^= OUT5;
	SetRelays();
	SwitchSettingsAltered = 1;
}

void
main(void)
{
	unsigned char	EeValue;
	
	InitHw();
	TIFR = 0x20;	// Clear ICF1
	TIMSK |= 0x20;	// On TICIE1
	TIMSK &= ~0x10;	// Off OCIE1A
	PowUpTmr = 0;
	MidiProgramm = 255;
	RelayState = OUT1;
	// Global enable interrupts
	#asm("sei")
	SetRelays();
	//
	while (1)
	{
		Wdog();
		if (TimFlg == 1)
		{
			TimFlg = 0;
			SetHwRelays();
			PrevFrontSwitchState = FrontSwitchState;
			FrontSwitchState = FrontSwitchMajority[0];
			if (((FrontSwitchState & IN1) != 0) &&
				((PrevFrontSwitchState & IN1) == 0))
			{
				SetOut1();
			}
			if (((FrontSwitchState & IN2) != 0) &&
				((PrevFrontSwitchState & IN2) == 0))
			{
				SetOut2();
			}
			if (((FrontSwitchState & IN3) != 0) &&
				((PrevFrontSwitchState & IN3) == 0))
			{
				SetOut3();
			}
			if (((FrontSwitchState & IN4) != 0) &&
				((PrevFrontSwitchState & IN4) == 0))
			{
				SetOut4();
			}
			if (((FrontSwitchState & IN5) != 0) &&
				((PrevFrontSwitchState & IN5) == 0))
			{
				SetOut5();
			}
		}
		if (DataReceived)
		{
			DataReceived = 0;
			if ((RecByte & IN1) != 0)
				SetOut1();
			if ((RecByte & IN2) != 0)
				SetOut2();
			if ((RecByte & IN3) != 0)
				SetOut3();
			if ((RecByte & IN4) != 0)
				SetOut4();
			if ((RecByte & IN5) != 0)
				SetOut5();
		}
		if (MidiProgramm != 255)
		{
			if (MidiProgramm == LastReceivedProgramm &&
				 SwitchSettingsAltered != 0)
			{
				if ((FrontSwitchState & IN6) == 0/*!= 0*/)
					EEPROM_write(MidiProgramm, RelayState);
			}
			else
			{
				EeValue =  EEPROM_read(MidiProgramm);
				if ((EeValue & ~(OUT1 | OUT2 | OUT3 | OUT4 | OUT5)) == 0)
				{
					RelayState = EeValue;
					SetRelays();
				}
			}
			LastReceivedProgramm = MidiProgramm;
			SwitchSettingsAltered = 0;
			MidiProgramm = 255;
		}	
	}
}
#include <mega8.h>

void
InitHw(void)
{
	// Input/Output Ports initialization
	// Port B initialization
	// Func7=Out Func6=Out Func5=Out Func4=Out Func3=Out Func2=Out Func1=Out Func0=In 
	// State7=0 State6=0 State5=0 State4=0 State3=0 State2=0 State1=0 State0=P 
	PORTB = 0x01;
	DDRB = 0xFE;
	// Port C initialization
	// Func6=In Func5=In Func4=In Func3=In Func2=In Func1=In Func0=In 
	// State6=P State5=P State4=P State3=P State2=P State1=P State0=P 
	PORTC = 0x7F;
	DDRC = 0x00;
	// Port D initialization
	// Func7=In Func6=In Func5=In Func4=In Func3=Out Func2=Out Func1=Out Func0=In 
	// State7=P State6=P State5=P State4=P State3=1 State2=1 State1=1 State0=P 
	PORTD = 0xFF;
	DDRD = 0x0E;
	// Timer/Counter 0 initialization
	// Clock source: System Clock
	// Clock value: 125,000 kHz
	TCCR0 = 0x03;
	TCNT0 = 0x00;
	// Timer/Counter 1 initialization
	// Clock source: System Clock
	// Clock value: 8000,000 kHz
	// Mode: Normal top=FFFFh
	// OC1A output: Discon.
	// OC1B output: Discon.
	// Noise Canceler: Off
	// Input Capture on Falling Edge
	// Timer 1 Overflow Interrupt: Off
	// Input Capture Interrupt: On
	// Compare A Match Interrupt: On
	// Compare B Match Interrupt: Off
	TCCR1A = 0x00;
	TCCR1B = 0x01;
	TCNT1H = 0x00;
	TCNT1L = 0x00;
	ICR1H = 0x00;
	ICR1L = 0x00;
	OCR1AH = 0x00;
	OCR1AL = 0x00;
	OCR1BH = 0x00;
	OCR1BL = 0x00;
	// Timer/Counter 2 initialization
	// Clock source: System Clock
	// Clock value: Timer 2 Stopped
	// Mode: Normal top=FFh
	// OC2 output: Disconnected
	ASSR = 0x00;
	TCCR2 = 0x00;
	TCNT2 = 0x00;
	OCR2 = 0x00;
	// External Interrupt(s) initialization
	// INT0: Off
	// INT1: Off
	MCUCR = 0x00;
	// Timer(s)/Counter(s) Interrupt(s) initialization
	TIMSK=0x31;
	// USART initialization
	// Communication Parameters: 8 Data, 1 Stop, No Parity
	// USART Receiver: On
	// USART Transmitter: On
	// USART Mode: Asynchronous
	// USART Baud rate: 31250
	UCSRA = 0x00;
	UCSRB = 0x98;
	UCSRC = 0x86;
	UBRRH = 0x00;
	UBRRL = 0x0F;
	// Analog Comparator initialization
	// Analog Comparator: Off
	// Analog Comparator Input Capture by Timer/Counter 1: Off
	ACSR = 0x80;
	SFIOR = 0x00;
	// Watchdog Timer initialization
	// Watchdog Timer Prescaler: OSC/2048k
#pragma optsize-
	WDTCR=0x1F;
	WDTCR=0x0F;
#ifdef _OPTIMIZE_SIZE_
#pragma optsize+
#endif
}
