/*****************************************************
Chip type           : ATmega8
Program type        : Application
Clock frequency     : 8,000000 MHz
Memory model        : Small
External SRAM size  : 0
Data Stack size     : 256
*****************************************************/

#include	<mega8.h>
#include	"Hw.h"

unsigned char	Ms10Tmr;
unsigned char	FrontSwitchMajority[4];
unsigned char	RelayState;
unsigned char	HwRelayState;
unsigned char	FrontSwitchState;
unsigned char	PrevFrontSwitchState;
unsigned char	DelayValue;
unsigned char	SecTmr;
unsigned char	PowUpTmr;
//
unsigned char	BitCnt;
unsigned char	RecByteI;
unsigned char	RecByte;
//
unsigned char	MidiChannel;
unsigned char	MidiProgramm;
unsigned char	LastReceivedProgramm;

bit	TimFlg;
bit	prgChangeDetectedBefore;
bit	SwitchSettingsAltered;
bit	DataReceived;

// USART Receiver interrupt service routine
interrupt [USART_RXC] void usart_rx_isr(void)
{
	unsigned char	inByte;

	inByte = UCSRA;
	inByte = UDR;

	if ((inByte & 0x80) != 0)
	{	// Command
		if ((inByte & 0xF0) == 0xC0)
		{
			if ((inByte & 0x0F) == MidiChannel)
			{
				prgChangeDetectedBefore = 1;
			}
		}
		else
		{
			prgChangeDetectedBefore = 0;
		}
	}
	else
	{	// Data
		if (prgChangeDetectedBefore != 0)
		{
			prgChangeDetectedBefore = 0;
			MidiProgramm = (inByte & 0x7F);
		}
	}
}

// USART Transmitter interrupt service routine
interrupt [USART_TXC] void usart_tx_isr(void)
{
	UCSRB &= ~0x40;
	TIFR = 0x20;	// Clear ICF1
	TIMSK |= 0x20;	// On TICIE1
}

// Timer 0 overflow interrupt service routine
interrupt [TIM0_OVF] void timer0_ovf_isr(void)
{	// 2 mS @ 8MHz
	if (DelayValue != 0)
		DelayValue--;
	if (Ms10Tmr != 0)
		Ms10Tmr--;
	else
	{	// 10 mS @ 8MHz
		Ms10Tmr = 4;
		FrontSwitchMajority[1] = FrontSwitchMajority[2];
		FrontSwitchMajority[2] = FrontSwitchMajority[3];
		FrontSwitchMajority[3] = (PINC & 0x3F) ^ 0x3F;
		FrontSwitchMajority[0] = ((FrontSwitchMajority[2] | FrontSwitchMajority[3]) & FrontSwitchMajority[1])
			| (FrontSwitchMajority[2] & FrontSwitchMajority[3]);
		MidiChannel = (PIND >> 4) ^ 0x0F;
		if (SecTmr != 0)
			SecTmr--;
		else
		{
			SecTmr = 99;
			if (PowUpTmr < 254)
				PowUpTmr++;
		}
		TimFlg = 1;
	}
}
//TIMSK/TIFR
//Bit 5  TICIE1
//Bit 4  OCIE1A
// Timer 1 input capture interrupt service routine
interrupt [TIM1_CAPT] void timer1_capt_isr(void)
{
	OCR1A = ICR1 + 384;
	BitCnt = 0;
	TIMSK &= ~0x20;	// Off TICIE1
	TIFR = 0x10;	// Clear OCF1A
	TIMSK |= 0x10;	// On OCIE1A
}

// Timer 1 output compare A interrupt service routine
interrupt [TIM1_COMPA] void timer1_compa_isr(void)
{
	OCR1A += 256;
	BitCnt++;
	if (BitCnt < 9)
	{
		RecByteI = RecByteI >> 1;
		if ((PINB & 0x01) != 0)
			RecByteI |= 0x80;
	}
	else
	{
		TIFR = 0x20;	// Clear ICF1
		TIMSK |= 0x20;	// On TICIE1
		TIMSK &= ~0x10;	// Off OCIE1A
		RecByte = RecByteI;
		DataReceived = 1;
	}
}

unsigned char
EEPROM_read(unsigned short Address)
{
	while(EECR & 0x02)	// Wait for completion of previous write
		;
	EEAR = Address;	// Set up address register
	EECR |= 0x01;	// Start eeprom read by writing EERE
	return EEDR;	// Return data from data register
}

void
EEPROM_write(unsigned short Address, unsigned char Data)
{
	if (EEPROM_read(Address) == Data)
	{
//		NextEeProgReq = 1;
		return;
	}
	EEAR = Address;	// Set up address and data registers
	EEDR = Data;
	#asm("cli")
	EECR |= 0x04;	// Write logical one to EEMWE
	EECR |= 0x0A;	// Start eeprom write by setting EEWE
	#asm("sei")
}

void
Wdog(void)
{
	#asm("wdr")
}

void
Waitms(unsigned char Del)
{
	DelayValue = Del;
	while (DelayValue != 0)
	{
		Wdog();
	}
}

void
SetHwRelays(void)
{
	
	HwRelayState = RelayState | 0x01;
	if (PowUpTmr > 10)
		HwRelayState |= OUT6;
	if (PowUpTmr > 15)
		HwRelayState |= OUT7;
	PORTB = HwRelayState;
}

void
SetRelays(void)
{
	SetHwRelays();
	UCSRA = 0x40;
	UCSRB |= 0x40;
	TIMSK &= ~0x20;	// off TICIE1
	TIMSK &= ~0x10;	// Off OCIE1A
	UDR = HwRelayState;
}

void
SetOut1(void)
{
	RelayState |= OUT1;
	SetHwRelays();
	Waitms(5);
	RelayState &= ~(OUT2 | OUT3 | OUT4);
	SetRelays();
	SwitchSettingsAltered = 1;
}

void
SetOut2(void)
{
	RelayState |= OUT2;
	SetHwRelays();
	Waitms(5);
	RelayState &= ~(OUT1 | OUT3 | OUT4);
	SetRelays();
	SwitchSettingsAltered = 1;
}

void
SetOut3(void)
{
	RelayState |= OUT3;
	SetHwRelays();
	Waitms(5);
	RelayState &= ~(OUT1 | OUT2 | OUT4);
	SetRelays();
	SwitchSettingsAltered = 1;
}

void
SetOut4(void)
{
	RelayState |= OUT4;
	SetHwRelays();
	Waitms(5);
	RelayState &= ~(OUT1 | OUT2 | OUT3);
	SetRelays();
	SwitchSettingsAltered = 1;
}

void
SetOut5(void)
{
	RelayState ^= OUT5;
	SetRelays();
	SwitchSettingsAltered = 1;
}

void
NextOut(void)
{
	if ((RelayState & OUT1) != 0)
		SetOut2();
	else if ((RelayState & OUT2) != 0)
		SetOut3();
	else if ((RelayState & OUT3) != 0)
		SetOut4();
	else
		SetOut1();
}

void
PrevOut(void)
{
	if ((RelayState & OUT1) != 0)
		SetOut4();
	else if ((RelayState & OUT2) != 0)
		SetOut1();
	else if ((RelayState & OUT3) != 0)
		SetOut2();
	else
		SetOut3();
}

void
main(void)
{
	unsigned char	EeValue;
	
	InitHw();
	TIFR = 0x20;	// Clear ICF1
	TIMSK |= 0x20;	// On TICIE1
	TIMSK &= ~0x10;	// Off OCIE1A
	PowUpTmr = 0;
	MidiProgramm = 255;
	RelayState = OUT1;
	// Global enable interrupts
	#asm("sei")
	Waitms(255);
	SetRelays();
	//
	while (1)
	{
		Wdog();
		if (TimFlg == 1)
		{
			TimFlg = 0;
			SetHwRelays();
			PrevFrontSwitchState = FrontSwitchState;
			FrontSwitchState = FrontSwitchMajority[0];
			if (((FrontSwitchState & IN1) != 0) &&
				((PrevFrontSwitchState & IN1) == 0))
			{
				NextOut();
			}
			if (((FrontSwitchState & IN2) != 0) &&
				((PrevFrontSwitchState & IN2) == 0))
			{
				PrevOut();
			}
			if (((FrontSwitchState & IN3) != 0) &&
				((PrevFrontSwitchState & IN3) == 0))
			{
				//SetOut3();
			}
			if (((FrontSwitchState & IN4) != 0) &&
				((PrevFrontSwitchState & IN4) == 0))
			{
				//SetOut4();
			}
			if (((FrontSwitchState & IN5) != 0) &&
				((PrevFrontSwitchState & IN5) == 0))
			{
				SetOut5();
			}
		}
		if (DataReceived)
		{
			DataReceived = 0;
			if ((RecByte & IN1) != 0)
				SetOut1();
			if ((RecByte & IN2) != 0)
				SetOut2();
			if ((RecByte & IN3) != 0)
				SetOut3();
			if ((RecByte & IN4) != 0)
				SetOut4();
			if ((RecByte & IN5) != 0)
				SetOut5();
		}
		if (MidiProgramm != 255)
		{
			if (MidiProgramm == LastReceivedProgramm &&
				 SwitchSettingsAltered != 0)
			{
				if ((FrontSwitchState & IN6) == 0/*!= 0*/)
					EEPROM_write(MidiProgramm, RelayState);
			}
			else
			{
				EeValue =  EEPROM_read(MidiProgramm);
				if ((EeValue & ~(OUT1 | OUT2 | OUT3 | OUT4 | OUT5)) == 0)
				{
					RelayState = EeValue;
					SetRelays();
				}
			}
			LastReceivedProgramm = MidiProgramm;
			SwitchSettingsAltered = 0;
			MidiProgramm = 255;
		}	
	}
}
