/*****************************************************
Chip type           : ATmega8
Program type        : Application
Clock frequency     : 8,000000 MHz
Memory model        : Small
External SRAM size  : 0
Data Stack size     : 256
*****************************************************/

#include	<mega8.h>
#include	"Hw.h"

unsigned char	Ms10Tmr;
unsigned char	FrontSwitchMajority[4];
unsigned char	HwRelayState;
unsigned char	FrontSwitchState;
unsigned char	PrevFrontSwitchState;
//
unsigned char	BitCnt;
unsigned char	RecByteI;
unsigned char	RecByte;
//

bit	TimFlg;
bit	DataReceived;

// USART Receiver interrupt service routine
interrupt [USART_RXC] void usart_rx_isr(void)
{
	unsigned char	inByte;

	inByte = UCSRA;
	inByte = UDR;
}

// USART Transmitter interrupt service routine
interrupt [USART_TXC] void usart_tx_isr(void)
{
	UCSRB &= ~0x40;
	TIFR = 0x20;	// Clear ICF1
	TIMSK |= 0x20;	// On TICIE1
}

// Timer 0 overflow interrupt service routine
interrupt [TIM0_OVF] void timer0_ovf_isr(void)
{	// 2 mS @ 8MHz
	if (Ms10Tmr != 0)
		Ms10Tmr--;
	else
	{	// 10 mS @ 8MHz
		Ms10Tmr = 4;
		FrontSwitchMajority[1] = FrontSwitchMajority[2];
		FrontSwitchMajority[2] = FrontSwitchMajority[3];
		FrontSwitchMajority[3] = (PINC & 0x3F) ^ 0x3F;
		FrontSwitchMajority[0] = ((FrontSwitchMajority[2] | FrontSwitchMajority[3]) & FrontSwitchMajority[1])
			| (FrontSwitchMajority[2] & FrontSwitchMajority[3]);
		TimFlg = 1;
	}
	if ((PINB & 0x01) != 0)
		PORTD |= 0x04;
	else
		PORTD &= ~0x04;
}
//TIMSK/TIFR
//Bit 5  TICIE1
//Bit 4  OCIE1A
// Timer 1 input capture interrupt service routine
interrupt [TIM1_CAPT] void timer1_capt_isr(void)
{
	PORTD &= ~0x08;
	OCR1A = ICR1 + 384;
	BitCnt = 0;
	TIMSK &= ~0x20;	// Off TICIE1
	TIFR = 0x10;	// Clear OCF1A
	TIMSK |= 0x10;	// On OCIE1A
}

// Timer 1 output compare A interrupt service routine
interrupt [TIM1_COMPA] void timer1_compa_isr(void)
{
	OCR1A += 256;
	BitCnt++;
	if (BitCnt < 9)
	{
		RecByteI = RecByteI >> 1;
		if ((PINB & 0x01) != 0)
			RecByteI |= 0x80;
	}
	else
	{
		TIFR = 0x20;	// Clear ICF1
		TIMSK |= 0x20;	// On TICIE1
		TIMSK &= ~0x10;	// Off OCIE1A
		RecByte = RecByteI;
		DataReceived = 1;
		PORTD |= 0x08;
	}
}

void
Wdog(void)
{
	#asm("wdr")
}

void
SetHwRelays(void)
{
	HwRelayState = RecByte | 0x01;
	PORTB = HwRelayState;
}

void
SendIn(unsigned char Val)
{
	UCSRA = 0x40;
	UCSRB |= 0x40;
	TIMSK &= ~0x20;	// off TICIE1
	TIMSK &= ~0x10;	// Off OCIE1A
	UDR = Val;
}

void
main(void)
{
	InitHw();
	TIFR = 0x20;	// Clear ICF1
	TIMSK |= 0x20;	// On TICIE1
	TIMSK &= ~0x10;	// Off OCIE1A
	// Global enable interrupts
	#asm("sei")
	while (1)
	{
		Wdog();
		if (TimFlg == 1)
		{
			TimFlg = 0;
			PrevFrontSwitchState = FrontSwitchState;
			FrontSwitchState = FrontSwitchMajority[0];
			if (((FrontSwitchState & IN1) != 0) &&
				((PrevFrontSwitchState & IN1) == 0))
			{
				SendIn(IN1);
			}
			if (((FrontSwitchState & IN2) != 0) &&
				((PrevFrontSwitchState & IN2) == 0))
			{
				SendIn(IN2);
			}
			if (((FrontSwitchState & IN3) != 0) &&
				((PrevFrontSwitchState & IN3) == 0))
			{
				SendIn(IN3);
			}
			if (((FrontSwitchState & IN4) != 0) &&
				((PrevFrontSwitchState & IN4) == 0))
			{
				SendIn(IN4);
			}
			if (((FrontSwitchState & IN5) != 0) &&
				((PrevFrontSwitchState & IN5) == 0))
			{
				SendIn(IN5);
			}
		}
		if (DataReceived)
		{
			DataReceived = 0;
			SetHwRelays();
		}
	}
}
#include <mega8.h>

void
InitHw(void)
{
	// Input/Output Ports initialization
	// Port B initialization
	// Func7=Out Func6=Out Func5=Out Func4=Out Func3=Out Func2=Out Func1=Out Func0=In 
	// State7=0 State6=0 State5=0 State4=0 State3=0 State2=0 State1=0 State0=P 
	PORTB = 0x01;
	DDRB = 0xFE;
	// Port C initialization
	// Func6=In Func5=In Func4=In Func3=In Func2=In Func1=In Func0=In 
	// State6=P State5=P State4=P State3=P State2=P State1=P State0=P 
	PORTC = 0x7F;
	DDRC = 0x00;
	// Port D initialization
	// Func7=In Func6=In Func5=In Func4=In Func3=Out Func2=Out Func1=Out Func0=In 
	// State7=P State6=P State5=P State4=P State3=1 State2=1 State1=1 State0=P 
	PORTD = 0xFF;
	DDRD = 0x0E;
	// Timer/Counter 0 initialization
	// Clock source: System Clock
	// Clock value: 125,000 kHz
	TCCR0 = 0x03;
	TCNT0 = 0x00;
	// Timer/Counter 1 initialization
	// Clock source: System Clock
	// Clock value: 8000,000 kHz
	// Mode: Normal top=FFFFh
	// OC1A output: Discon.
	// OC1B output: Discon.
	// Noise Canceler: Off
	// Input Capture on Falling Edge
	// Timer 1 Overflow Interrupt: Off
	// Input Capture Interrupt: On
	// Compare A Match Interrupt: On
	// Compare B Match Interrupt: Off
	TCCR1A = 0x00;
	TCCR1B = 0x01;
	TCNT1H = 0x00;
	TCNT1L = 0x00;
	ICR1H = 0x00;
	ICR1L = 0x00;
	OCR1AH = 0x00;
	OCR1AL = 0x00;
	OCR1BH = 0x00;
	OCR1BL = 0x00;
	// Timer/Counter 2 initialization
	// Clock source: System Clock
	// Clock value: Timer 2 Stopped
	// Mode: Normal top=FFh
	// OC2 output: Disconnected
	ASSR = 0x00;
	TCCR2 = 0x00;
	TCNT2 = 0x00;
	OCR2 = 0x00;
	// External Interrupt(s) initialization
	// INT0: Off
	// INT1: Off
	MCUCR = 0x00;
	// Timer(s)/Counter(s) Interrupt(s) initialization
	TIMSK=0x31;
	// USART initialization
	// Communication Parameters: 8 Data, 1 Stop, No Parity
	// USART Receiver: On
	// USART Transmitter: On
	// USART Mode: Asynchronous
	// USART Baud rate: 31250
	UCSRA = 0x00;
	UCSRB = 0x98;
	UCSRC = 0x86;
	UBRRH = 0x00;
	UBRRL = 0x0F;
	// Analog Comparator initialization
	// Analog Comparator: Off
	// Analog Comparator Input Capture by Timer/Counter 1: Off
	ACSR = 0x80;
	SFIOR = 0x00;
	// Watchdog Timer initialization
	// Watchdog Timer Prescaler: OSC/2048k
#pragma optsize-
	WDTCR=0x1F;
	WDTCR=0x0F;
#ifdef _OPTIMIZE_SIZE_
#pragma optsize+
#endif
}
