/*****************************************************
Chip type           : ATmega8
Program type        : Application
Clock frequency     : 8,000000 MHz
Memory model        : Small
External SRAM size  : 0
Data Stack size     : 256
*****************************************************/

#include	<mega8.h>
#include	"Hw.h"

unsigned char	Ms10Tmr;
unsigned char	FrontSwitchMajority[4];
unsigned char	HwRelayState;
unsigned char	FrontSwitchState;
unsigned char	PrevFrontSwitchState;
//
unsigned char	BitCnt;
unsigned char	RecByteI;
unsigned char	RecByte;
//

bit	TimFlg;
bit	DataReceived;

// USART Receiver interrupt service routine
interrupt [USART_RXC] void usart_rx_isr(void)
{
	unsigned char	inByte;

	inByte = UCSRA;
	inByte = UDR;
}

// USART Transmitter interrupt service routine
interrupt [USART_TXC] void usart_tx_isr(void)
{
	UCSRB &= ~0x40;
	TIFR = 0x20;	// Clear ICF1
	TIMSK |= 0x20;	// On TICIE1
}

// Timer 0 overflow interrupt service routine
interrupt [TIM0_OVF] void timer0_ovf_isr(void)
{	// 2 mS @ 8MHz
	if (Ms10Tmr != 0)
		Ms10Tmr--;
	else
	{	// 10 mS @ 8MHz
		Ms10Tmr = 4;
		FrontSwitchMajority[1] = FrontSwitchMajority[2];
		FrontSwitchMajority[2] = FrontSwitchMajority[3];
		FrontSwitchMajority[3] = (PINC & 0x3F) ^ 0x3F;
		FrontSwitchMajority[0] = ((FrontSwitchMajority[2] | FrontSwitchMajority[3]) & FrontSwitchMajority[1])
			| (FrontSwitchMajority[2] & FrontSwitchMajority[3]);
		TimFlg = 1;
	}
	if ((PINB & 0x01) != 0)
		PORTD |= 0x04;
	else
		PORTD &= ~0x04;
}
//TIMSK/TIFR
//Bit 5  TICIE1
//Bit 4  OCIE1A
// Timer 1 input capture interrupt service routine
interrupt [TIM1_CAPT] void timer1_capt_isr(void)
{
	PORTD &= ~0x08;
	OCR1A = ICR1 + 384;
	BitCnt = 0;
	TIMSK &= ~0x20;	// Off TICIE1
	TIFR = 0x10;	// Clear OCF1A
	TIMSK |= 0x10;	// On OCIE1A
}

// Timer 1 output compare A interrupt service routine
interrupt [TIM1_COMPA] void timer1_compa_isr(void)
{
	OCR1A += 256;
	BitCnt++;
	if (BitCnt < 9)
	{
		RecByteI = RecByteI >> 1;
		if ((PINB & 0x01) != 0)
			RecByteI |= 0x80;
	}
	else
	{
		TIFR = 0x20;	// Clear ICF1
		TIMSK |= 0x20;	// On TICIE1
		TIMSK &= ~0x10;	// Off OCIE1A
		RecByte = RecByteI;
		DataReceived = 1;
		PORTD |= 0x08;
	}
}

void
Wdog(void)
{
	#asm("wdr")
}

void
SetHwRelays(void)
{
	HwRelayState = RecByte | 0x01;
	PORTB = HwRelayState;
}

void
SendIn(unsigned char Val)
{
	UCSRA = 0x40;
	UCSRB |= 0x40;
	TIMSK &= ~0x20;	// off TICIE1
	TIMSK &= ~0x10;	// Off OCIE1A
	UDR = Val;
}

void
main(void)
{
	InitHw();
	TIFR = 0x20;	// Clear ICF1
	TIMSK |= 0x20;	// On TICIE1
	TIMSK &= ~0x10;	// Off OCIE1A
	// Global enable interrupts
	#asm("sei")
	while (1)
	{
		Wdog();
		if (TimFlg == 1)
		{
			TimFlg = 0;
			PrevFrontSwitchState = FrontSwitchState;
			FrontSwitchState = FrontSwitchMajority[0];
			if (((FrontSwitchState & IN1) != 0) &&
				((PrevFrontSwitchState & IN1) == 0))
			{
				SendIn(IN1);
			}
			if (((FrontSwitchState & IN2) != 0) &&
				((PrevFrontSwitchState & IN2) == 0))
			{
				SendIn(IN2);
			}
			if (((FrontSwitchState & IN3) != 0) &&
				((PrevFrontSwitchState & IN3) == 0))
			{
				SendIn(IN3);
			}
			if (((FrontSwitchState & IN4) != 0) &&
				((PrevFrontSwitchState & IN4) == 0))
			{
				SendIn(IN4);
			}
			if (((FrontSwitchState & IN5) != 0) &&
				((PrevFrontSwitchState & IN5) == 0))
			{
				SendIn(IN5);
			}
		}
		if (DataReceived)
		{
			DataReceived = 0;
			SetHwRelays();
		}
	}
}
